/* This file is in the public domain */

#include <string>

#include <opencl/filters.h>
#include <opencl/symkey.h>
#include <opencl/lookup.h>

#include <opencl/seal.h>
using namespace OpenCL_types;

OpenCL::Filter* lookup_block(const std::string&,
                             const OpenCL::BlockCipher_Key&);
OpenCL::Filter* lookup_mode(const std::string&, const OpenCL::BlockCipher_Key&,
                            const OpenCL::SymmetricKey&);
OpenCL::Filter* lookup_stream(const std::string&,
                              const OpenCL::StreamCipher_Key&);
OpenCL::Filter* lookup_hash(const std::string&);
OpenCL::Filter* lookup_mac(const std::string&, const OpenCL::MAC_Key&);
OpenCL::Filter* lookup_rng(const std::string&);
OpenCL::Filter* lookup_encoder(const std::string&);
OpenCL::Filter* lookup_block(const std::string&, const OpenCL::SymmetricKey&);
OpenCL::Filter* lookup_s2k(const std::string&, const OpenCL::SymmetricKey&);

OpenCL::Filter* lookup(const std::string& algname, const std::string& hex_key,
                       const std::string& hex_iv)
   {
   OpenCL::SymmetricKey key(hex_key), iv(hex_iv);
   OpenCL::Filter* filter;

   filter = lookup_block(algname, key);
   if(filter) return filter;

   filter = lookup_mode(algname, key, iv);
   if(filter) return filter;

   filter = lookup_stream(algname, key);
   if(filter) return filter;

   filter = lookup_hash(algname);
   if(filter) return filter;

   filter = lookup_mac(algname, key);
   if(filter) return filter;

   filter = lookup_rng(algname);
   if(filter) return filter;

   filter = lookup_encoder(algname);
   if(filter) return filter;

   filter = lookup_s2k(algname, key);
   if(filter) return filter;

   return 0;
   }

OpenCL::Filter* lookup_hash(const std::string& algname)
   {
   OpenCL::HashFunction* hash = 0;
   try {
     hash = OpenCL::get_hash(algname);
   }
   catch(OpenCL::Algorithm_Not_Found) {}

   if(hash)
      return new OpenCL::Hash_Filter(hash);

   return 0;
   }

OpenCL::Filter* lookup_stream(const std::string& algname,
                              const OpenCL::SymmetricKey& key)
   {
   OpenCL::StreamCipher* cipher = 0;
   try {
     cipher = OpenCL::get_stream_cipher(algname);
   }
   catch(OpenCL::Algorithm_Not_Found) {}

   if(cipher)
      return new OpenCL::StreamCipher_Filter(cipher, key);

   /*
   These specific variants are requested by name for the validation tests

   FIXME: There should be some way to handle this in the lookup code... I think

   At the very least, parse the <X,Y> and have only one call!
   */
   else if(algname == "SEAL<0,8>")
      return new OpenCL::StreamCipher_Filter(new OpenCL::SEAL(0, 8), key);
   else if(algname == "SEAL<0x013577AF,4>")
      return new OpenCL::StreamCipher_Filter(new OpenCL::SEAL(0x013577AF, 4),
                                             key);

   else return 0;
   }
