/* This file is in the public domain */

#include <string>

#include <opencl/filters.h>
#include <opencl/lookup.h>

#include <opencl/des.h>
#include <opencl/square.h>
#include <opencl/md2.h>
#include <opencl/md5.h>
#include <opencl/rmd128.h>
#include <opencl/rmd160.h>
#include <opencl/sha1.h>
#include <opencl/emac.h>
#include <opencl/hmac.h>
using namespace OpenCL;

Filter* lookup_mac(const std::string& algname, const SymmetricKey& key)
   {
   MessageAuthenticationCode* mac = 0;
   try {
     mac = get_mac(algname);
   }
   catch(Algorithm_Not_Found) {}

   if(mac)
      return new MAC_Filter(mac, key);

   // These are cases which *should* be handled by lookup
   if(algname == "EMAC<Square>")
      return new MAC_Filter(new EMAC(new Square), key);
   else if(algname == "EMAC<DES>")
      return new MAC_Filter(new EMAC(new DES), key);

   else if(algname == "HMAC-MD2")
      return new MAC_Filter(new HMAC(new MD2), key);
   else if(algname == "HMAC-MD5")
      return new MAC_Filter(new HMAC(new MD5), key);
   else if(algname == "HMAC-SHA1")
      return new MAC_Filter(new HMAC(new SHA1), key);
   else if(algname == "HMAC-RIPEMD-128")
      return new MAC_Filter(new HMAC(new RIPEMD_128), key);
   else if(algname == "HMAC-RIPEMD-160")
      return new MAC_Filter(new HMAC(new RIPEMD_160), key);
   else
      return 0;
   }
