/*************************************************
* Miscellaneous PK Functions Header File         *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PUBKEY_MISC_H__
#define OPENCL_PUBKEY_MISC_H__

#include <opencl/opencl.h>

namespace OpenCL {

/*************************************************
* Encoding Method for Encryption                 *
*************************************************/
class EME
   {
   public:
      virtual u32bit maximum_input_size(u32bit) const = 0;

      SecureVector<byte> encode(const byte[], u32bit, u32bit) const;
      SecureVector<byte> encode(const SecureVector<byte>&, u32bit) const;
      SecureVector<byte> decode(const byte[], u32bit, u32bit) const;
      SecureVector<byte> decode(const SecureVector<byte>&, u32bit) const;

      virtual ~EME() {}
   private:
      virtual SecureVector<byte> pad(const byte[], u32bit, u32bit) const = 0;
      virtual SecureVector<byte> unpad(const byte[], u32bit, u32bit) const = 0;
   };

/*************************************************
* Encoding Method for Signatures, Appendix       *
*************************************************/
class EMSA
   {
   public:
      SecureVector<byte> encode(const byte[], u32bit, u32bit) const;
      SecureVector<byte> encode(const SecureVector<byte>&, u32bit) const;

      bool is_valid(const byte[], u32bit, const byte[], u32bit,
                    u32bit) const throw();
      bool is_valid(const SecureVector<byte>&, const SecureVector<byte>&,
                    u32bit) const throw();

      virtual ~EMSA() {}
   private:
      virtual SecureVector<byte> encode_op(const byte[], u32bit,
                                           u32bit) const = 0;
      virtual bool verify(const byte[], u32bit,
                          const byte[], u32bit, u32bit) const;
   };

/*************************************************
* Mask Generation Function Base Class            *
*************************************************/
class MaskGenerationFunction
   {
   public:
      virtual void mask(const byte[], u32bit, byte[], u32bit) const = 0;
      virtual ~MaskGenerationFunction() {}
   };

/*************************************************
* Key Derivation Function Base Class             *
*************************************************/
class KeyDerivationFunction
   {
   public:
      virtual SymmetricKey derive(const byte[], u32bit, u32bit) const = 0;
      virtual ~KeyDerivationFunction() {}
   };

}

#endif
