/*************************************************
* Public Key Interface Header File               *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_PUBKEY_H__
#define OPENCL_PUBKEY_H__

#include <opencl/opencl.h>
#include <opencl/pk_keys.h>
#include <opencl/pk_misc.h>

namespace OpenCL {

/*************************************************
* Public Key Encryptor                           *
*************************************************/
class PK_Encryptor
   {
   public:
      SecureVector<byte> encrypt(const byte[], u32bit) const;
      SecureVector<byte> encrypt(const SecureVector<byte>&) const;
      u32bit maximum_input_size() const;
      PK_Encryptor(const PK_Encrypting_Key&, EME* = 0);
      ~PK_Encryptor() { delete encoder; }
   private:
      const PK_Encrypting_Key& key;
      const EME* encoder;
   };

/*************************************************
* Public Key Decryptor                           *
*************************************************/
class PK_Decryptor
   {
   public:
      SecureVector<byte> decrypt(const byte[], u32bit) const;
      SecureVector<byte> decrypt(const SecureVector<byte>&) const;
      PK_Decryptor(const PK_Decrypting_Key&, EME* = 0);
      ~PK_Decryptor() { delete encoder; }
   private:
      const PK_Decrypting_Key& key;
      const EME* encoder;
   };

/*************************************************
* Public Key Signer                              *
*************************************************/
class PK_Signer
   {
   public:
      SecureVector<byte> sign_message(const byte[], u32bit);
      void update(const byte[], u32bit);
      SecureVector<byte> signature();
      PK_Signer(const PK_Signing_Key&, HashFunction* = 0, EMSA* = 0);
      ~PK_Signer() { delete hash; delete encoder; }
   private:
      const PK_Signing_Key& key;
      HashFunction* hash;
      EMSA* encoder;
      SecureVector<byte> message;
   };

/*************************************************
* Public Key Verifier Base Class                 *
*************************************************/
class PK_Verifier
   {
   public:
      bool verify_message(const byte[], u32bit, const byte[], u32bit);
      void update(const byte[], u32bit);
      bool valid_signature(const byte[], u32bit);
      PK_Verifier(HashFunction* = 0, EMSA* = 0);
      virtual ~PK_Verifier() { delete hash; delete encoder; }
   protected:
      virtual bool validate_signature(const SecureVector<byte>&,
                                      const byte[], u32bit) = 0;
      EMSA* encoder;
   private:
      HashFunction* hash;
      SecureVector<byte> message;
   };

/*************************************************
* PK_Verifier_with_MR                            *
*************************************************/
class PK_Verifier_with_MR : public PK_Verifier
   {
   public:
      PK_Verifier_with_MR(const PK_Verifying_with_MR_Key&, HashFunction* = 0,
                          EMSA* = 0);
   private:
      bool validate_signature(const SecureVector<byte>&, const byte[], u32bit);
      const PK_Verifying_with_MR_Key& key;
   };

/*************************************************
* PK_Verifier_wo_MR                              *
*************************************************/
class PK_Verifier_wo_MR : public PK_Verifier
   {
   public:
      PK_Verifier_wo_MR(const PK_Verifying_wo_MR_Key&, HashFunction* = 0);
   private:
      bool validate_signature(const SecureVector<byte>&, const byte[], u32bit);
      const PK_Verifying_wo_MR_Key& key;
   };


}

#endif
