/*************************************************
* ECB Mode Source File                           *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/ecb.h>

namespace OpenCL {

/*************************************************
* ECB Encryption Constructor                     *
*************************************************/
ECB::ECB(BlockCipher* c, const BlockCipher_Key& key, const std::string& n) :
   BlockCipherMode(c, n, key, BlockCipherModeIV())
   {
   }

/*************************************************
* Encrypt in ECB mode                            *
*************************************************/
void ECB::write(const byte input[], u32bit length)
   {
   buffer.copy(position, input, length);
   if(position + length >= BLOCK_SIZE)
      {
      do_block(buffer, buffer);
      send(buffer, BLOCK_SIZE);
      input += (BLOCK_SIZE - position);
      length -= (BLOCK_SIZE - position);
      while(length >= BLOCK_SIZE)
         {
         do_block(input, buffer);
         send(buffer, BLOCK_SIZE);
         input += BLOCK_SIZE;
         length -= BLOCK_SIZE;
         }
      buffer.copy(input, length);
      position = 0;
      }
   position += length;
   }

/*************************************************
* Finish encrypting in ECB mode                  *
*************************************************/
void ECB::end_msg()
   {
   if(position)
      throw Exception(name() + ": input was not in full blocks");
   }

}
