/*************************************************
* EMSA3 Source File                              *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/emsa3.h>
#include <opencl/ieee1363.h>

namespace OpenCL {

/*************************************************
* EMSA3 Encode Operation                         *
*************************************************/
SecureVector<byte> EMSA3::encode_op(const byte msg[], u32bit msg_len,
                                    u32bit output_length) const
   {
   output_length /= 8;

   if(msg_len != HASH_OUTPUT_LENGTH)
      throw Invalid_Argument("EMSA3::pad: Bad input length");
   if(output_length < hash_id.size() + HASH_OUTPUT_LENGTH + 10)
      throw Invalid_Argument("EMSA3::pad: Output length is too small");

   SecureVector<byte> T(output_length);
   const u32bit P_LENGTH = output_length - HASH_OUTPUT_LENGTH -
                           hash_id.size() - 2;

   T[0] = 0x01;
   set_mem(T+1, P_LENGTH, 0xFF);
   T[P_LENGTH+1] = 0x00;
   T.copy(P_LENGTH+2, hash_id, hash_id.size());
   T.copy(output_length-HASH_OUTPUT_LENGTH, msg, msg_len);
   return T;
   }

/*************************************************
* EMSA3 Constructor                              *
*************************************************/
EMSA3::EMSA3(HashFunction* hash) : HASH_OUTPUT_LENGTH(hash->OUTPUT_LENGTH)
   {
   hash_id = pkcs_hash_id(hash);
   if(hash_id.size() == 0)
      throw Invalid_Argument("EMSA3 cannot be used with " + hash->name());
   delete hash;
   }

}
