/*************************************************
* Initalization Function Source File             *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/init.h>
#include <opencl/allocate.h>
#include <opencl/rng.h>
#include <opencl/mutex.h>
#include <opencl/timers.h>
#include <opencl/randpool.h>

#ifdef OPENCL_EXT_MUTEX_PTHREAD
#include <opencl/mux_pthr.h>
#endif


#ifdef OPENCL_EXT_TIMER_HARDWARE
#include <opencl/tm_hard.h>
#endif

#ifdef OPENCL_EXT_TIMER_POSIX
#include <opencl/tm_posix.h>
#endif

#ifdef OPENCL_EXT_TIMER_UNIX
#include <opencl/tm_unix.h>
#endif

namespace OpenCL {

namespace {

/*************************************************
* Check if an option is set in the argument      *
*************************************************/
bool arg_set(const std::string& args, const std::string& option)
   {
   if(args.find(option) != std::string::npos)
      return true;
   return false;
   }

/*************************************************
* Register a mutex type, if possible             *
*************************************************/
bool set_mutex()
   {
#ifdef OPENCL_EXT_MUTEX_PTHREAD
   set_mutex_type(new Pthread_Mutex);
   return true;
#endif

   return false;
   }

/*************************************************
* Register a high resoultion timer, if possible  *
*************************************************/
void set_timer()
   {
#ifdef OPENCL_EXT_TIMER_HARDWARE
   set_timer_type(new Hardware_Timer);
   return;
#endif

#ifdef OPENCL_EXT_TIMER_POSIX
   set_timer_type(new POSIX_Timer);
   return;
#endif

#ifdef OPENCL_EXT_TIMER_UNIX
   set_timer_type(new Unix_Timer);
   return;
#endif
   }

}

/*************************************************
* Libary Initalization                           *
*************************************************/
LibraryInitializer::LibraryInitializer(const std::string& args)
   {
   void load_algorithms_list();
   void initalize_memory_subsystem();

   if(arg_set(args, "thread_safe"))
      if(!set_mutex())
         throw Exception("LibraryInitializer: thread safety impossible");

   if(!arg_set(args, "no_timers"))
      set_timer();

   initalize_memory_subsystem();
   load_algorithms_list();
   set_global_rng(new Randpool);
   }

/*************************************************
* Libary Deinitialization                        *
*************************************************/
LibraryInitializer::~LibraryInitializer()
   {
   void destroy_lookup_tables();

   set_global_rng(0);
   destroy_lookup_tables();
   set_timer_type(0);
   set_mutex_type(0);
   }

}
