/*************************************************
* Miscellaneous PK Functions Source File         *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#include <opencl/pk_misc.h>

namespace OpenCL {

/*************************************************
* Encode a message                               *
*************************************************/
SecureVector<byte> EME::encode(const byte msg[], u32bit msg_len,
                               u32bit key_bits) const
   {
   return pad(msg, msg_len, key_bits);
   }

/*************************************************
* Encode a message                               *
*************************************************/
SecureVector<byte> EME::encode(const SecureVector<byte>& msg,
                               u32bit key_bits) const
   {
   return pad(msg, msg.size(), key_bits);
   }

/*************************************************
* Decode a message                               *
*************************************************/
SecureVector<byte> EME::decode(const byte msg[], u32bit msg_len,
                               u32bit key_bits) const
   {
   return unpad(msg, msg_len, key_bits);
   }

/*************************************************
* Decode a message                               *
*************************************************/
SecureVector<byte> EME::decode(const SecureVector<byte>& msg,
                               u32bit key_bits) const
   {
   return unpad(msg, msg.size(), key_bits);
   }

/*************************************************
* Signature encoding                             *
*************************************************/
SecureVector<byte> EMSA::encode(const byte msg[], u32bit msg_len,
                                u32bit key_len) const
   {
   return encode_op(msg, msg_len, key_len);
   }

/*************************************************
* Signature encoding                             *
*************************************************/
SecureVector<byte> EMSA::encode(const SecureVector<byte>& msg,
                                u32bit key_len) const
   {
   return encode_op(msg, msg.size(), key_len);
   }

/*************************************************
* Signature decoding                             *
*************************************************/
bool EMSA::is_valid(const byte msg[], u32bit msg_len,
                    const byte hash[], u32bit hash_len,
                    u32bit key_len) const throw()
   {
   try {
      return verify(msg, msg_len, hash, hash_len, key_len);
   }
   catch(...)
      {
      return false;
      }
   }

/*************************************************
* Signature decoding                             *
*************************************************/
bool EMSA::is_valid(const SecureVector<byte>& msg,
                    const SecureVector<byte>& hash,
                    u32bit key_len) const throw()
   {
   try {
      return verify(msg, msg.size(), hash, hash.size(), key_len);
   }
   catch(...)
      {
      return false;
      }
   }

/*************************************************
* Default signature decoding                     *
*************************************************/
bool EMSA::verify(const byte msg[], u32bit msg_len,
                  const byte hash[], u32bit hash_len,
                  u32bit key_len) const
   {
   try {
      SecureVector<byte> encoded = encode(hash, hash_len, key_len);

      if(encoded.size() != msg_len)
         return false;

      return (cmp_mem(encoded.ptr(), msg, encoded.size()) == 0);
      }
   catch(Invalid_Argument)
      {
      return false;
      }
   }

}
